@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of urlwatch (https://thp.io/2008/urlwatch/).
-# Copyright (c) 2008-2018 Thomas Perl <m@thp.io>
+# Copyright (c) 2008-2019 Thomas Perl <m@thp.io>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -118,6 +118,7 @@ class UrlwatchCommand:
 
     def test_filter(self):
         job = self._find_job(self.urlwatch_config.test_filter)
+        job = job.with_defaults(self.urlwatcher.config_storage.config)
         if job is None:
             print('Not found: %r' % (self.urlwatch_config.test_filter,))
             return 1
@@ -128,6 +129,8 @@ class UrlwatchCommand:
 
         job_state = JobState(self.urlwatcher.cache_storage, job)
         job_state.process()
+        if job_state.exception is not None:
+            raise job_state.exception
         print(job_state.new_data)
         # We do not save the job state or job on purpose here, since we are possibly modifying the job
         # (ignore_cached) and we do not want to store the newly-retrieved data yet (filter testing)
@@ -150,7 +153,8 @@ class UrlwatchCommand:
             filters = [v for k, v in items if k == 'filter']
             items = [(k, v) for k, v in items if k != 'filter']
             d = {k: v for k, v in items}
-            d['filter'] = ','.join(filters)
+            if filters:
+                d['filter'] = ','.join(filters)
 
             job = JobBase.unserialize(d)
             print('Adding %r' % (job,))
@@ -217,6 +221,26 @@ class UrlwatchCommand:
             print('\nChat up your bot here: https://t.me/{}'.format(info['result']['username']))
             sys.exit(0)
 
+    def check_test_slack(self):
+        if self.urlwatch_config.test_slack:
+            config = self.urlwatcher.config_storage.config['report'].get('slack', None)
+            if not config:
+                print('You need to configure slack in your config first (see README.md)')
+                sys.exit(1)
+
+            webhook_url = config.get('webhook_url', None)
+            if not webhook_url:
+                print('You need to set up your slack webhook_url first (see README.md)')
+                sys.exit(1)
+
+            info = requests.post(webhook_url, json={"text": "Test message from urlwatch, your configuration is working"})
+            if info.status_code == requests.codes.ok:
+                print('Successfully sent message to Slack')
+                sys.exit(0)
+            else:
+                print('Error while submitting message to Slack:{0}'.format(info.text))
+                sys.exit(1)
+
     def check_smtp_login(self):
         if self.urlwatch_config.smtp_login:
             config = self.urlwatcher.config_storage.config['report']['email']
@@ -243,7 +267,7 @@ class UrlwatchCommand:
 
             smtp_username = smtp_config.get('user', config['from'])
             if not smtp_username:
-                print('Please configure the SMTP username in the config first.')
+                print('Please configure the SMTP user in the config first.')
                 success = False
 
             if not success:
@@ -265,6 +289,7 @@ class UrlwatchCommand:
         self.check_edit_config()
         self.check_smtp_login()
         self.check_telegram_chats()
+        self.check_test_slack()
         self.handle_actions()
         self.urlwatcher.run_jobs()
         self.urlwatcher.close()
