@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of urlwatch (https://thp.io/2008/urlwatch/).
-# Copyright (c) 2008-2018 Thomas Perl <m@thp.io>
+# Copyright (c) 2008-2019 Thomas Perl <m@thp.io>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -35,8 +35,10 @@ import os
 import imp
 import html.parser
 import hashlib
+import json
 
 from enum import Enum
+from lxml import etree
 
 from .util import TrackSubClasses
 
@@ -183,6 +185,19 @@ class Ical2TextFilter(FilterBase):
         return ical2text(data)
 
 
+class JsonFormatFilter(FilterBase):
+    """Convert to formatted json"""
+
+    __kind__ = 'format-json'
+
+    def filter(self, data, subfilter=None):
+        indentation = 4
+        if subfilter is not None:
+            indentation = int(subfilter)
+        parsed_json = json.loads(data)
+        return json.dumps(parsed_json, sort_keys=True, indent=indentation, separators=(',', ': '))
+
+
 class GrepFilter(FilterBase):
     """Filter only lines matching a regular expression"""
 
@@ -349,3 +364,86 @@ class HexdumpFilter(FilterBase):
         return '\n'.join('%s  %s' % (' '.join('%02x' % c for c in block),
                                      ''.join((chr(c) if (c > 31 and c < 127) else '.')
                                              for c in block)) for block in blocks)
+
+
+class LxmlParser:
+    EXPR_NAMES = {'css': 'a CSS selector',
+                  'xpath': 'an XPath expression'}
+
+    def __init__(self, filter_kind, subfilter, expr_key):
+        self.filter_kind = filter_kind
+        self.expression, self.method = self.parse_subfilter(
+            filter_kind, subfilter, expr_key, self.EXPR_NAMES[filter_kind])
+        self.parser = (etree.HTMLParser if self.method == 'html' else etree.XMLParser)()
+        self.data = ''
+
+    @staticmethod
+    def parse_subfilter(filter_kind, subfilter, expr_key, expr_name):
+        if subfilter is None:
+            raise ValueError('Need %s for filtering' % (expr_name,))
+        if isinstance(subfilter, str):
+            expression = subfilter
+            method = 'html'
+        elif isinstance(subfilter, dict):
+            if expr_key not in subfilter:
+                raise ValueError('Need %s for filtering' % (expr_name,))
+            expression = subfilter[expr_key]
+            method = subfilter.get('method', 'html')
+            if method not in ('html', 'xml'):
+                raise ValueError('%s method must be "html" or "xml", got %r' % (filter_kind, method))
+        else:
+            raise ValueError('%s subfilter must be a string or dict' % (filter_kind,))
+
+        return expression, method
+
+    def feed(self, data):
+        self.data += data
+
+    def _to_string(self, element):
+        # Handle "/text()" selector, which returns lxml.etree._ElementUnicodeResult (Issue #282)
+        if isinstance(element, str):
+            return element
+
+        return etree.tostring(element, pretty_print=True, method=self.method, encoding='unicode', with_tail=False)
+
+    def _get_filtered_elements(self):
+        try:
+            root = etree.fromstring(self.data, self.parser)
+        except ValueError:
+            # Strip XML declaration, for example: '<?xml version="1.0" encoding="utf-8"?>'
+            # for https://heronebag.com/blog/index.xml, an error happens, as we get a
+            # a (Unicode) string, but the XML contains its own "encoding" declaration
+            self.data = re.sub(r'^<[?]xml[^>]*[?]>', '', self.data)
+            # Retry parsing with XML declaration removed (Fixes #281)
+            root = etree.fromstring(self.data, self.parser)
+        if root is None:
+            return []
+        if self.filter_kind == 'css':
+            return root.cssselect(self.expression)
+        elif self.filter_kind == 'xpath':
+            return root.xpath(self.expression)
+
+    def get_filtered_data(self):
+        return '\n'.join(self._to_string(element) for element in self._get_filtered_elements())
+
+
+class CssFilter(FilterBase):
+    """Filter XML/HTML using CSS selectors"""
+
+    __kind__ = 'css'
+
+    def filter(self, data, subfilter=None):
+        lxml_parser = LxmlParser('css', subfilter, 'selector')
+        lxml_parser.feed(data)
+        return lxml_parser.get_filtered_data()
+
+
+class XPathFilter(FilterBase):
+    """Filter XML/HTML using XPath expressions"""
+
+    __kind__ = 'xpath'
+
+    def filter(self, data, subfilter=None):
+        lxml_parser = LxmlParser('xpath', subfilter, 'path')
+        lxml_parser.feed(data)
+        return lxml_parser.get_filtered_data()
